from __future__ import annotations

MAX_RESPONSE_SIZE = 50_000  # ~50KB JSON threshold

ENV_OURO_API_KEY = "OURO_API_KEY"
ENV_OURO_BASE_URL = "OURO_BASE_URL"
ENV_OURO_DATABASE_URL = "OURO_DATABASE_URL"
ENV_OURO_DATABASE_ANON_KEY = "OURO_DATABASE_ANON_KEY"

DEFAULT_HTTP_PORT = 8000
